@@ -7,9 +7,26 @@ import torchquantum.functional as tqf
 from torchquantum.macro import C_DTYPE
 from typing import Union, List, Iterable
 
+import functools
 
 __all__ = ['QuantumState']
 
+# def _register_non_param_gate(f):
+#     def gate(self, wires, inverse, comp_method):
+#         self.op_list.append({
+#             'name': f.__name__.lower(),
+#             'has_params': module.has_params,
+#             'trainable': module.trainable,
+#             'wires': module.wires,
+#             'n_wires': module.n_wires,
+#             'params': params
+#         )
+#
+#     return reg_gate
+# #
+# def _register_param_gate(f):
+#     @functools.wraps(f)
+
 
 class QuantumState(nn.Module):
     """
@@ -34,14 +51,17 @@ class QuantumState(nn.Module):
         _state = torch.reshape(_state, [2] * self.n_wires)
         self.register_buffer('state', _state)
 
-        self.states = None
+        repeat_times = [bsz] + [1] * len(self.state.shape)
+        self._states = self.state.repeat(*repeat_times)
+        self.register_buffer('states', self._states)
 
-        self.reset_states(bsz)
+        self.op_list = []
 
     def clone_states(self, existing_states: torch.Tensor):
         self.states = existing_states.clone()
 
-    def set_states(self, states: torch.Tensor):
+    def set_states(self, states: Union[torch.Tensor, List]):
+        states = torch.tensor(states, dtype=C_DTYPE).to(self.state.device)
         bsz = states.shape[0]
         self.states = torch.reshape(states, [bsz] + [2] * self.n_wires)
 
@@ -77,7 +97,7 @@ class QuantumState(nn.Module):
         return self.__class__.__name__
 
     def __repr__(self):
-        return f"{self.name} {self.n_wires} wires \n state: {self.states}"
+        return f"{self.name} {self.n_wires} wires \n state: {self.get_states_1d()}"
 
     def hadamard(self,
                  wires: Union[List[int], int],
@@ -611,9 +631,9 @@ class QuantumState(nn.Module):
                 comp_method: str = 'bmm'):
 
         if isinstance(params, Iterable):
-            params = torch.Tensor(params)
+            params = torch.tensor(params, dtype=C_DTYPE)
         else:
-            params = torch.Tensor([params])
+            params = torch.tensor([params], dtype=C_DTYPE)
 
         tqf.qubitunitary(self,
                     wires=wires,
@@ -628,9 +648,9 @@ class QuantumState(nn.Module):
                 comp_method: str = 'bmm'):
 
         if isinstance(params, Iterable):
-            params = torch.Tensor(params)
+            params = torch.tensor(params, dtype=C_DTYPE)
         else:
-            params = torch.Tensor([params])
+            params = torch.tensor([params], dtype=C_DTYPE)
 
         tqf.qubitunitaryfast(self,
                     wires=wires,
@@ -645,9 +665,9 @@ class QuantumState(nn.Module):
                 comp_method: str = 'bmm'):
 
         if isinstance(params, Iterable):
-            params = torch.Tensor(params)
+            params = torch.tensor(params, dtype=C_DTYPE)
         else:
-            params = torch.Tensor([params])
+            params = torch.tensor([params], dtype=C_DTYPE)
 
         tqf.qubitunitarystrict(self,
                     wires=wires,
